module Samscript
  class Screen
    java_import 'java.awt.Toolkit'
    java_import 'java.awt.Rectangle'
    java_import 'java.awt.Robot'

    DefaultParams = { match: true,
                      tolerance: 0,
                      interval: 100,
                      timeout: 1000 }

    def self.resolution
      @tk ||= Java::JavaAwt::Toolkit.get_default_toolkit
      @tk.get_screen_size
    end

    def self.capture area = resolution
      rect = Rectangle.new area
      @robot ||= Robot.new
      @robot.create_screen_capture rect
    end

    def self.wait_for_pixel params = {}
      %w(location colour).map(&:to_sym).each do |sym|
        raise "invalid params" if params[sym].nil?
      end

      x, y      = params[:location][0], params[:location][1]
      colour    = params[:colour]
      tolerance = params[:tolerance] || DefaultParams[:tolerance]
      match     = params[:match]     || DefaultParams[:match]
      interval  = params[:interval]  || DefaultParams[:interval]
      timeout   = params[:timeout]   || DefaultParams[:timeout]
      attempts  = (timeout/interval+1).floor

      return if match == :any or tolerance >= 255

      attempts.times do
        screen = self.capture
        pixel = Samscript::Image.get_pixel(screen, x: x, y: y)
        if Samscript::Image.get_difference(pixel, colour) <= tolerance
          yield(true) if block_given?
          return
        end

        sleep(interval.to_f/1000)
      end

      yield(false) if block_given?
    end
  end
end
